#!/bin/bash

# This is the script that will be placed onto the USB flash drive, that will
# run on bootup to install the software onto the device.

# I want us to error out if we use an undefined variable, so we will catch errors.
set -u

# Read in our config file, if it exists

# How big, in megabytes, to make the nand flash image
nand_image_size=512

# How big, in megabytes, to make the ext3 File System on the Live Read/Write
# USB image
ext3fs_size=100

# How big, in megabytes, to make the boot partition for HD installation image
boot_partition_size=128

# Options for swap partition: 0 = No swap, 1 = swap file created (but not
# enabled because it is NOT put into /etc/fstab) this could be useful for
# hibernation,  2 = swap on (swap file created and put into /etc/fstab)
swap_option=2

# How big, in megabytes, to make the swap partition for HD installation image
# Only meaningful when swap_option != 0, if swap_option == 0, then
# swap_partition_size will be forced to 0.
swap_partition_size=256

# How big, in megabytes, to make the FAT32 partition for HD installation image
fat32_partition_size=0

#################### usplash functions start ####################################
SPLASHWRITE=0
# Determine if we have usplash_write available
type usplash_write > /dev/null 2>&1 && SPLASHWRITE=1

# Disable usplash, since we want text mode
SPLASHWRITE=0

# show the progress at status bar.
# $1 = 0-100
splash_progress(){
    splash_write "PROGRESS $1"
    return 0
}
# display the text no matter whether verbose is set or not
splash_display(){
    echo "$@"
    splash_write "TEXT-URGENT $@"
    return 0
}
# set the splash delay time
splash_delay(){
    splash_write "TIMEOUT $1"
    return 0
}
# call the usplash_write command, if enabled
splash_write(){
    if [ "${SPLASHWRITE}" -eq 1 ]
    then
        usplash_write "$@"
    else:
	echo "ussplash_write: $@"
    fi
}
####################### usplash functions end ###############################





splash_delay 200
splash_display 'INSTALL..........'

pre_scsi_disk_number=$( ls /sys/class/scsi_disk | wc -l)
found=no
# Find the install disk
mount -t proc none /proc
while true; do
      for device in 'hda' 'hdb' 'sda' 'sdb'
      do
        echo "checking device: /dev/${device} for installation target"
        if [ -e /sys/block/${device}/removable ]; then
           if [ "$(cat /sys/block/${device}/removable)" = "0" ]; then
              if cat /proc/mounts | grep /dev/${device}
              then
                  continue
              else
                  found="yes"
                  splash_display "found harddisk at /dev/${device}"
                  break
              fi
           fi
         fi
      done
      if [ "$found" = "yes" ]; then
        break;
      fi
      /bin/sleep 5
      echo "Did not find an installation target device"
done
echo "This script will delete all partitions on the disk /dev/${device}"
echo "All data will be lost!!"
echo "Are you sure you want to continue? (if yes, type YES, anything else othereise)"
echo
read answer
if [ "$answer" != "YES" ]; then
	exit 1
fi

echo "We will install to /dev/${device}"
blocks=`fdisk -s /dev/${device}`
cylinders=$((blocks*2/63/255))

splash_display "Deleting Partition Table on /dev/${device} ..."
splash_delay 200
dd if=/dev/zero of=/dev/${device} bs=512 count=2
sync
splash_progress 5
splash_delay 10

splash_display "Creating New Partiton Table on /dev/${device} ..."
splash_delay 200

swapoff -a
splash_delay 200

PART=`mktemp`
sfdisk /dev/${device}  << EOF
,$((boot_partition_size*1000/8192)),L
,$((swap_partition_size*1000/8192)),S
;
EOF

sync
mount /dev/${device}1 /.asjdflasjdf
splash_progress 10
splash_delay 10
splash_display "Formatting /dev/${device}1 w/ ext3..."
splash_delay 200
mkfs.ext3 -q /dev/${device}1
sync
splash_progress 20
splash_delay 10
splash_display "Formatting /dev/${device}3 w/ ext3..."
splash_delay 200
mkfs.ext3 -q /dev/${device}3
sync
splash_progress 60
splash_delay 10
if [ $swap_partition_size -ne 0 ]
then
    splash_display "Formatting /dev/${device}2 w/ swap..."
    splash_delay 1000
    mkswap /dev/${device}2
    if [ $fat32_partition_size -ne 0 ]
    then
       splash_display "Formatting /dev/${device}4 w/ vfat..."
       splash_delay 1000
       mkfs.vfat -q /dev/${device}4
    fi
else if [ $fat32_partition_size -ne 0 ]
     then
       splash_display "Formatting /dev/${device}3 w/ vfat..."
       splash_delay 1000
       mkfs.vfat -q /dev/${device}3
    fi
fi
sync
splash_progress 65
splash_delay 10
splash_display 'Mounting partitions...'
splash_delay 200

MNTDIR=`mktemp -d`
FSDIR=`mktemp -d`

mount /dev/${device}3 $MNTDIR
mkdir $MNTDIR/boot
mount /dev/${device}1 $MNTDIR/boot
splash_progress 70
splash_delay 10

splash_display 'Copying system files onto hard disk drive...'
splash_delay 200
echo "Copying root filesystem..."
mkdir $FSDIR/squash
mkdir $FSDIR/root
mount -o loop -t squashfs /mnt/live/LiveOS/squashfs.img $FSDIR/squash
mount -o loop  $FSDIR/squash/LiveOS/ext3fs.img $FSDIR/root
splash_display 'Copying system ROOT onto hard disk drive...'
cp -av $FSDIR/root/. $MNTDIR


/usr/sbin/chroot $MNTDIR /usr/lib/moblin-installer/tasks/hard/100_create_grub.sh "quiet root=/dev/${device}3"
/sbin/grub-install --root-directory=$MNTDIR /dev/${device}

echo "/dev/${device}1 /boot   ext3    defaults        1 1" >> $MNTDIR/etc/fstab
echo "/dev/${device}2 swap  swap  defaults        0 0" >> $MNTDIR/etc/fstab

/usr/sbin/chroot $MNTDIR /usr/lib/moblin-installer/tasks/hard/800_superreadahead.sh
/usr/sbin/chroot $MNTDIR /usr/lib/moblin-installer/tasks/hard/810_disable_services.sh
/usr/sbin/chroot $MNTDIR /usr/lib/moblin-installer/tasks/hard/500_devices.sh
/usr/sbin/chroot $MNTDIR /usr/lib/moblin-installer/tasks/hard/910_cleanup.sh

splash_progress 90
splash_delay 10
splash_display 'Unmounting partitions...'
splash_delay 200


umount $MNTDIR/boot
umount $MNTDIR
umount  $FSDIR/root
umount  $FSDIR/squash

splash_progress 95
splash_delay 10
sleep 1
splash_delay 6000
splash_display "Install Successfully"
splash_display "Unplug USB Key, System Will Reboot Automatically"

# need to call reboot --help and let file system cache hold it, since we will
# unplug USB disk soon, and after that, reboot command will not be accessible.
# The reason why reboot still works sometimes without this is the whole
# "rootfs.img" is cached when it is copied to HD. But when rootfs.img become
# bigger and bigger the whole "rootfs.img" will not be able to fully cached (we
# have found this issue when creating big installation)
reboot --help > /dev/null 2>&1

while [ $pre_scsi_disk_number = $(ls /sys/class/scsi_disk | wc -l) ]
do
    sleep 1
done

splash_progress 100
splash_delay 1

reboot -f
